package cd.casic.ci.process.api.engine.common;

import cn.hutool.core.date.DateUtil;

import java.util.Random;
import java.util.Set;

/**
 * @author by mianbin
 * @Classname VMUtils
 * @Description TODO
 * @Date 2025/3/31 16:22
 */
public class VMUtils {

    /**
     * 生成阶段ID
     *
     * @param seq 序号
     * @return 生成的阶段ID
     */
    public static String genStageId(int seq) {
        return "stage-" + seq;
    }

    /**
     * 为用户生成阶段ID
     *
     * @param seq 序号
     * @return 生成的用户阶段ID
     */
    public static String genStageIdForUser(int seq) {
        return "stage_" + seq;
    }

    /**
     * 生成停止虚拟机任务ID
     *
     * @param seq 序号
     * @return 生成的停止虚拟机任务ID
     */
    public static String genStopVMTaskId(int seq) {
        return getStopVmLabel() + seq;
    }

    /**
     * 生成结束点任务ID
     *
     * @param seq 序号
     * @return 生成的结束点任务ID
     */
    public static String genEndPointTaskId(int seq) {
        return getEndLabel() + seq;
    }

    /**
     * 生成虚拟机任务序号
     *
     * @param containerSeq 容器序号
     * @param taskSeq      任务序号
     * @return 生成的虚拟机任务序号
     */
    public static int genVMTaskSeq(int containerSeq, int taskSeq) {
        return containerSeq * 1000 + taskSeq;
    }

    /**
     * 生成矩阵容器序号
     *
     * @param matrixGroupId 矩阵组ID
     * @param innerIndex    内部索引
     * @return 生成的矩阵容器序号
     */
    public static int genMatrixContainerSeq(int matrixGroupId, int innerIndex) {
        return matrixGroupId * 1000 + innerIndex;
    }

    /**
     * 生成矩阵作业ID
     *
     * @param groupJobId 组作业ID
     * @param innerSeq   内部序号
     * @return 生成的矩阵作业ID
     */
    public static String genMatrixJobId(String groupJobId, int innerSeq) {
        return groupJobId + "." + innerSeq;
    }

    /**
     * 生成启动虚拟机任务ID
     *
     * @param containerSeq 容器序号
     * @return 生成的启动虚拟机任务ID
     */
    public static String genStartVMTaskId(String containerSeq) {
        return getStartVmLabel() + containerSeq;
    }

    /**
     * 获取停止虚拟机标签
     *
     * @return 停止虚拟机标签
     */
    public static String getStopVmLabel() {
        return "stopVM-";
    }

    /**
     * 获取清理虚拟机标签
     *
     * @return 清理虚拟机标签
     */
    public static String getCleanVmLabel() {
        return "Clean_Job#";
    }

    /**
     * 获取启动虚拟机标签
     *
     * @return 启动虚拟机标签
     */
    public static String getStartVmLabel() {
        return "startVM-";
    }

    /**
     * 获取准备虚拟机标签
     *
     * @return 准备虚拟机标签
     */
    public static String getPrepareVmLabel() {
        return "Prepare_Job#";
    }

    /**
     * 获取等待标签
     *
     * @return 等待标签
     */
    public static String getWaitLabel() {
        return "Wait_Finish_Job#";
    }

    /**
     * 获取结束标签
     *
     * @return 结束标签
     */
    public static String getEndLabel() {
        return "end-";
    }

    /**
     * 获取容器作业ID
     *
     * @param randomSeed 随机种子
     * @param jobIdSet   已存在的作业ID集合
     * @return 生成的容器作业ID
     */
    public static String getContainerJobId(int randomSeed, Set<String> jobIdSet) {
        Random random = new Random(randomSeed);
        StringBuilder sequence = new StringBuilder();
        for (int i = 0; i < 3; i++) {
            sequence.append((char) (random.nextInt('z' - 'A' + 1) + 'A'));
        }
        String jobId = "job_" + sequence;
        return jobIdSet.contains(jobId) ? jobId + "_" + DateUtil.current() : jobId;
    }

    /**
     * 判断是否为虚拟机任务
     *
     * @param taskId 任务ID
     * @return 如果是虚拟机任务返回 true，否则返回 false
     */
    public static boolean isVMTask(String taskId) {
        return taskId.startsWith(getStartVmLabel()) ||
                taskId.startsWith(getStopVmLabel()) ||
                taskId.startsWith(getEndLabel());
    }

    /**
     * 判断是否为矩阵容器ID
     *
     * @param containerId 容器ID
     * @return 如果是矩阵容器ID返回 true，否则返回 false
     */
    public static boolean isMatrixContainerId(String containerId) {
        try {
            return Integer.parseInt(containerId) > 1000;
        } catch (NumberFormatException ignore) {
            return false;
        }
    }
}
