package cd.casic.ci.log.scm.api.scm;

import cd.casic.ci.log.scm.dal.pojo.RevisionInfo;
import cd.casic.ci.log.scm.enums.ScmType;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.ci.log.scm.enums.CodeSvnRegion;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;


@RestController
@Tag(name = "仓库分支、tag、hook相关")
@RequestMapping("/service/scm")
public interface ServiceScmResource {

    @Operation(summary = "Get the repo latest revision")
    @GetMapping("/latestRevision")
    @Parameters({
            @Parameter(name = "projectName", description = "项目名称", required = true),
            @Parameter(name = "url", description = "仓库地址", required = true),
            @Parameter(name = "type", description = "仓库类型", required = true),
            @Parameter(name = "branchName", description = "branch name", required = false),
            @Parameter(name = "additionalPath", description = "SVN additional path", required = false),
            @Parameter(name = "privateKey", description = "privateKey", required = false),
            @Parameter(name = "passPhrase", description = "passPhrase", required = false),
            @Parameter(name = "token", description = "token", required = false),
            @Parameter(name = "region", description = "仓库区域前缀(只有svn用到)", required = false),
            @Parameter(name = "userName", description = "仓库对应的用户名", required = true)
    })
    CommonResult<RevisionInfo> getLatestRevision(
            @RequestParam
            String projectName,
            @RequestParam
            String url,
            @RequestParam
            ScmType type,
            @RequestParam
            String branchName,
            @RequestParam
            String additionalPath,
            @RequestParam
            String privateKey,
            @RequestParam
            String passPhrase,
            @RequestParam
            String token,
            @RequestParam
            CodeSvnRegion region,
            @RequestParam
            String userName
    );

    @Operation(summary = "添加Git或者Gitlab WEB hook")
    @PostMapping("/addWebhook")
    @Parameters({
            @Parameter(name = "projectName", description = "项目名称", required = true),
            @Parameter(name = "url", description = "仓库地址", required = true),
            @Parameter(name = "type", description = "仓库类型", required = true),
            @Parameter(name = "privateKey", description = "privateKey", required = true),
            @Parameter(name = "passPhrase", description = "passPhrase", required = false),
            @Parameter(name = "token", description = "token", required = true),
            @Parameter(name = "region", description = "仓库区域前缀(只有svn用到)", required = false),
            @Parameter(name = "userName", description = "仓库对应的用户名", required = true),
            @Parameter(name = "event", description = "事件类型", required = false),
            @Parameter(name = "hookUrl", description = "回调Url", required = false)
    })
    CommonResult<Boolean> addWebhook(
            @RequestParam
            String projectName,
            @RequestParam
            String url,
            @RequestParam
            ScmType type,
            @RequestParam
            String privateKey,
            @RequestParam
            String passPhrase,
            @RequestParam
            String token,
            @RequestParam
            CodeSvnRegion region,
            @RequestParam
            String userName,
            @RequestParam
            String event,
            @RequestParam
            String hookUrl
    );
}
