package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.sensitive.SensitiveApiApplyReq;
import cd.casic.pipeline.store.pojo.common.sensitive.SensitiveApiInfo;
import cd.casic.pipeline.store.pojo.common.sensitive.SensitiveApiNameInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "USER_SDK_SENSITIVEAPI", description = "用户-敏感API")
@RestController
@RequestMapping("/user/sdk/{storeType}/{storeCode}/sensitiveApi/")
public interface UserSensitiveApiResource {

    @Operation(summary = "获取未审批API列表")
    @GetMapping("unApprovalApiList")
    CommonResult<List<SensitiveApiNameInfo>> unApprovalApiList(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "开发语言", required = true)
            @RequestParam("language")
            String language
    );

    @Operation(summary = "敏感API申请")
    @PostMapping("apply")
    CommonResult<Boolean> apply(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "申请API请求体")
            SensitiveApiApplyReq sensitiveApiApplyReq
    );

    @Operation(summary = "查看敏感API列表")
    @GetMapping("list")
    PageResult<SensitiveApiInfo> list(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "API名", required = false)
            @RequestParam("apiName")
            String apiName,
            @Parameter(description = "第几页", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页条数", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "取消申请敏感API")
    @PutMapping("cancel/{id")
    CommonResult<Boolean> cancel(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "敏感API名称", required = true)
            @PathVariable("id")
            String id
    );
}