package cd.casic.pipeline.store.api.image;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.image.response.ImageDetail;
import cd.casic.pipeline.store.pojo.image.response.ImageRepoInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;
import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID_DEFAULT_VALUE;

@Tag(name = "SERVICE_MARKET_IMAGE", description = "SERVICE-研发商店-镜像")
@RestController
@RequestMapping("/service/market")
public interface ServiceStoreImageResource {

    @Operation(summary = "查询镜像是否已安装到项目")
    @GetMapping("/image/projectCodes/{projectCode}/imageCodes/{imageCode}/isInstalled")
    CommonResult<Boolean> isInstalled(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像标识", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "镜像标识", required = true)
            @PathVariable("imageCode")
            String imageCode
    );

    @Operation(summary = "根据code查询镜像详情")
    @GetMapping("/image/projectCodes/{projectCode}/imageCodes/{imageCode}/imageVersions/{imageVersion}")
    CommonResult<ImageRepoInfo> getImageRepoInfoByCodeAndVersion(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目标识", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "镜像标识", required = true)
            @PathVariable("imageCode")
            String imageCode,
            @Parameter(description = "镜像版本", required = false)
            @PathVariable("imageVersion")
            String imageVersion,
            @Parameter(description = "流水线Id", required = true)
            @RequestParam("pipelineId")
            String pipelineId,
            @Parameter(description = "构建Id", required = true)
            @RequestParam("buildId")
            String buildId
    );

    @Operation(summary = "根据镜像代码获取镜像信息")
    @GetMapping("/imageCodes/{imageCode}")
    CommonResult<ImageDetail> getImagesByCodeAndVersion(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像代码", required = true)
            @PathVariable("imageCode")
            String imageCode,
            @Parameter(description = "版本号", required = false)
            @RequestParam("version")
            String version
    );

    @Operation(summary = "获取所有的自研公共镜像")
    @GetMapping("/image/self_develop/public_images")
    CommonResult<List<ImageRepoInfo>> getSelfDevelopPublicImages();

    @Operation(summary = "根据code和版本号查询镜像状态")
    @GetMapping("/image/imageCodes/{imageCode}/imageVersions/{imageVersion}/imageStatus")
    CommonResult<String> getImageStatusByCodeAndVersion(
            @Parameter(description = "镜像标识", required = true)
            @PathVariable("imageCode")
            String imageCode,
            @Parameter(description = "镜像版本", required = false)
            @PathVariable("imageVersion")
            String imageVersion
    );
}